export default function (options) {
    options = {
        label: 'Copy',
        ...options
    };
    return {
        transform: doc => {
            let hasCodeBlock = false;
            doc.querySelectorAll('pre > code').forEach((code) => {
                const pre = code.closest('pre');
                const button = doc.createElement('button');
                button.setAttribute('type', 'button');
                button.classList.add('copy-code-button');
                button.textContent = options.label;
                pre.append(button);
                hasCodeBlock = true;
            });
            if (hasCodeBlock) {
                const script = doc.createElement('script');
                script.type = 'module';
                script.textContent = `
          document.addEventListener('click', event => {
            const button = event.target.closest('.copy-code-button');
            const pre = button?.closest('pre');
            const code = pre?.querySelector('code');
            if (button && code) {
              navigator.clipboard.writeText(code.innerText);
            }
          });
        `;
                doc.body.append(script);
            }
            return doc;
        }
    };
}
