import Prism from 'prismjs';
import PrismLoader from 'prismjs/components/index.js';
PrismLoader('diff');
PrismLoader.silent = true;
export function highlight(code, language) {
    const alias = language.replace(/^diff-/, '');
    const isDiff = /^diff-/i.test(language);
    if (!Prism.languages[alias]) {
        PrismLoader(alias);
        if (!Prism.languages[alias]) {
            throw new Error(`Unsupported language for code highlighting: "${language}"`);
        }
    }
    if (isDiff) {
        Prism.languages[language] = Prism.languages.diff;
    }
    return Prism.highlight(code, Prism.languages[language], language);
}
export default function (options) {
    options = {
        ignoreMissingLangs: true,
        ...options
    };
    return {
        transform: doc => {
            doc.querySelectorAll('pre > code[class]').forEach((code) => {
                code.classList.forEach(className => {
                    if (className.startsWith('language-')) {
                        const language = className.replace(/^language-/, '');
                        try {
                            code.innerHTML = highlight(code.textContent ?? '', language);
                        }
                        catch (err) {
                            if (!options.ignoreMissingLangs) {
                                throw new Error(err.message);
                            }
                        }
                    }
                });
            });
            return doc;
        }
    };
}
