import { randomUUID } from 'crypto';
import { createSlug } from '../../utilities/slug.js';
function getHeadingLevel(level) {
    return parseInt(level.slice(1));
}
function makeListItem(doc, heading) {
    const hasId = heading.hasAttribute('id');
    const hasChildWithId = heading.querySelector('[id]');
    const label = heading.textContent ?? '';
    let id;
    if (hasId)
        id = heading.getAttribute('id');
    if (hasChildWithId)
        id = heading.querySelector('[id]')?.getAttribute('id');
    if (!id) {
        id = createSlug(label);
        if (!id)
            id = randomUUID().slice(-12);
        heading.setAttribute('id', id);
    }
    const li = doc.createElement('li');
    const a = doc.createElement('a');
    a.setAttribute('href', `#${id}`);
    a.textContent = label;
    li.append(a);
    return li;
}
function makeNav(doc, headings, container, level) {
    let heading;
    do {
        heading = headings.shift();
    } while (heading && !/^h[123456]$/i.test(heading.tagName));
    if (heading) {
        const currentLevel = getHeadingLevel(heading.tagName);
        let count = 0;
        let li;
        if (currentLevel < level) {
            do {
                container = container.parentNode.parentNode;
                count--;
            } while (count > currentLevel - level);
        }
        else if (currentLevel > level) {
            do {
                li = container.lastChild;
                if (li === null)
                    li = container.appendChild(makeListItem(doc, heading));
                container = li.appendChild(doc.createElement('ul'));
                count++;
            } while (count < currentLevel - level);
        }
        li = container.appendChild(makeListItem(doc, heading));
        makeNav(doc, headings, container, level + count);
    }
}
export default function (options) {
    options = {
        target: '.table-of-contents',
        startLevel: 'h2',
        endLevel: 'h4',
        within: 'body',
        ...options
    };
    return {
        transform: doc => {
            const within = options.within ? doc.querySelector(options.within) : doc.body;
            const startLevel = getHeadingLevel(options.startLevel);
            const endLevel = getHeadingLevel(options.endLevel);
            const levelsToInclude = [1, 2, 3, 4, 5, 6].slice(startLevel - 1, endLevel);
            const headingSelector = levelsToInclude.map(level => `h${level}`).join(',');
            if (!within)
                return doc;
            const headings = [...within.querySelectorAll(headingSelector)];
            const nav = doc.createElement('nav');
            makeNav(doc, headings, nav, startLevel);
            if (nav.children.length > 0) {
                [...doc.querySelectorAll(options.target)].forEach((container) => {
                    container.innerHTML = nav.outerHTML;
                });
            }
            return doc;
        }
    };
}
