import fs from 'fs/promises';
import frontMatter from 'front-matter';
import MarkdownIt from 'markdown-it';
import markdownItAttrs from 'markdown-it-attrs';
import markdownItContainer from 'markdown-it-container';
const markdown = MarkdownIt({
    html: true,
    xhtmlOut: false,
    breaks: false,
    langPrefix: 'language-',
    linkify: false,
    typographer: false,
    quotes: '“”‘’'
});
markdown.use(markdownItAttrs, {
    leftDelimiter: '{',
    rightDelimiter: '}',
    allowedAttributes: ['class', 'id', 'style', 'title', /^data-/]
});
['tip', 'warning'].forEach(type => {
    markdown.use(markdownItContainer, type, {
        render: function (tokens, idx) {
            if (tokens[idx].nesting === 1) {
                return `<div role="alert" class="callout callout--${type}">`;
            }
            return '</div>\n';
        }
    });
});
markdown.use(markdownItContainer, 'details', {
    validate: (params) => params.trim().match(/^details\s+(.*)$/),
    render: (tokens, idx) => {
        const m = tokens[idx].info.trim().match(/^details\s+(.*)$/);
        if (tokens[idx].nesting === 1) {
            return `<details>\n<summary>${markdown.utils.escapeHtml(m[1])}</summary>\n`;
        }
        return '</details>\n';
    }
});
export async function parse(file) {
    const data = await fs.readFile(file, 'utf8');
    const { attributes, body } = frontMatter(data);
    return {
        frontMatter: attributes,
        content: body
    };
}
export function render(content) {
    return markdown.render(content);
}
